require 'pathname'
require 'fileutils'
require 'indentation'
require_relative 'core_ext/string'

module Trekyll

    # Navigation Manager gets as imputs:
    # @page_name ( String with/without "prefix" )
    # @nav_dir_name ( name of directory with nav.yml file ) eg. _data/nav.yml

    # Navigation Manager sets as outputs:
    # @page_file_name ( sanitized page name for href/url creation )
    # @page_name ( String without "prefix" )
    # @page_type ( page/subpage )

    # Prefix definition:
    #   [#]     - Container
    #   #       - Sub page
    #   #[#]    - Sub page conainer
    #   ##      - Sub sub page
    #   ##[#]   - Sub sub container page
    #   ###     - Sub sub sub page
    #   [/#]    - Container with ommited page url (links to homepage when clicked)
    #   [!]     - Ommited page from navigation

    class NavigationManager

        CONFIGURABLE_PROPERTIES = [
            :page_name,
            :page_type,
            :nav_dir_name,
            :page_file_name
        ]

        attr_accessor *CONFIGURABLE_PROPERTIES

        # constructor *( defoult navigation location _data dir)
        def initialize(dir="_data")
            @nav_dir_name = dir
        end

        # Add page to nav.yml
        def add_page(page)
            # get page name
            @page_name = page

            # create a file name (remove special chars and convert spaces to "_")
            @page_file_name = @page_name.sanitize_as_page_name

            File.open( @nav_dir_name + '/nav.yml', 'a') do |f|
                # Get substring prefix  ##[#] | MyPage_name
                i = @page_name.index(/[a-zA-Z\p{L}]/)
                prefix = @page_name[0,i]
                c = prefix.count("#")

                # Define different spacing for first # and odd numbr of # (hashes)
                if c == 1
                    c = 0
                elsif c % 2 != 0
                    c = c+1
                end

                if prefix.include?("[#]")
                    f.puts "- title: \"#{@page_name.sub!(prefix,'')}\" ".indent(c)
                    f.puts "  href: /#{@page_file_name}/".indent(c)
                    f.puts "  subcategories:".indent(c)
                    @page_type = "page"

                elsif prefix.include?("[/#]")
                    f.puts "- title: \"#{@page_name.sub!(prefix,'')}\" ".indent(c)
                    f.puts "  subcategories:".indent(c)
                    @page_type = "page"

                elsif prefix.include?("#")
                    f.puts "- title: \"#{@page_name.sub!(prefix,'')}\" ".indent(c+2)
                    f.puts "  href: /#{@page_file_name}/".indent(c+2)
                    @page_type = "subpage"

                elsif @page_name.start_with?("[!]")
                    @page_name.sub!(prefix,"")
                else
                    f.puts "- title: \"#{page_name}\" "
                    f.puts "  href: /#{page_file_name}/"
                    @page_type = "page"
                end
            end
        end
    end
end